@@ -5,7 +5,7 @@ from distutils.core import setup, Extension
 
 _helpers_mod = Extension('_helpers',
                          sources=['_helpers.c', 'bupsplit.c'],
-                         depends=['../../config/config.h'])
+                         depends=['../../config/config.h', 'bupsplit.h'])
 
 setup(name='_helpers',
       version='0.1',
